@@ -38,6 +38,7 @@ def assemble(text_program):
     labels = {}
     instructions = []
     sideset_count = 0
+    sideset_enable = 0
     for line in text_program.split("\n"):
         line = line.strip()
         if not line:
@@ -55,6 +56,7 @@ def assemble(text_program):
             pass
         elif line.startswith(".side_set"):
             sideset_count = int(line.split()[1])
+            sideset_enable = 1 if "opt" in line else 0
         elif line.endswith(":"):
             label = line[:-1]
             if label in labels:
@@ -64,7 +66,7 @@ def assemble(text_program):
             # Only add as an instruction if the line isn't empty
             instructions.append(line)
 
-    max_delay = 2 ** (5 - sideset_count) - 1
+    max_delay = 2 ** (5 - sideset_count - sideset_enable) - 1
     assembled = []
     for instruction in instructions:
         # print(instruction)
@@ -76,10 +78,13 @@ def assemble(text_program):
                 raise RuntimeError("Delay too long:", delay)
             instruction.pop()
         if len(instruction) > 1 and instruction[-2] == "side":
+            if sideset_count == 0:
+                raise RuntimeError("No side_set count set")
             sideset_value = int(instruction[-1])
             if sideset_value > 2 ** sideset_count:
                 raise RuntimeError("Sideset value too large")
-            delay |= sideset_value << (5 - sideset_count)
+            delay |= sideset_value << (5 - sideset_count - sideset_enable)
+            delay |= sideset_enable << 4
             instruction.pop()
             instruction.pop()
 
@@ -98,7 +103,12 @@ def assemble(text_program):
                 raise SyntaxError(f"Invalid jmp target {repr(target)}")
 
             if len(instruction) > 2:
-                assembled[-1] |= CONDITIONS.index(instruction[1]) << 5
+                try:
+                    assembled[-1] |= CONDITIONS.index(instruction[1]) << 5
+                except ValueError as exc:
+                    raise ValueError(
+                        f"Invalid jmp condition '{instruction[1]}'"
+                    ) from exc
 
         elif instruction[0] == "wait":
             #                instr delay p sr index
@@ -146,7 +156,10 @@ def assemble(text_program):
             source = instruction[-1]
             source_split = mov_splitter(source)
             if len(source_split) == 1:
-                assembled[-1] |= MOV_SOURCES.index(source)
+                try:
+                    assembled[-1] |= MOV_SOURCES.index(source)
+                except ValueError as exc:
+                    raise ValueError(f"Invalid mov source '{source}'") from exc
             else:
                 assembled[-1] |= MOV_SOURCES.index(source_split[1])
                 if source[:1] == "!":
@@ -178,7 +191,10 @@ def assemble(text_program):
         elif instruction[0] == "set":
             #                instr delay dst data
             assembled.append(0b111_00000_000_00000)
-            assembled[-1] |= SET_DESTINATIONS.index(instruction[1]) << 5
+            try:
+                assembled[-1] |= SET_DESTINATIONS.index(instruction[1]) << 5
+            except ValueError as exc:
+                raise ValueError(f"Invalid set destination '{instruction[1]}'") from exc
             value = int(instruction[-1])
             if not 0 <= value <= 31:
                 raise RuntimeError("Set value out of range")
@@ -186,6 +202,6 @@ def assemble(text_program):
         else:
             raise RuntimeError("Unknown instruction:" + instruction[0])
         assembled[-1] |= delay << 8
-    # print(hex(assembled[-1]))
+        # print(bin(assembled[-1]))
 
     return array.array("H", assembled)
